  
/*
*  C Implementation: Gateway_Client
*
* Copyright (C) -2007  MªCruz Olivas Delgado <maoldel@teleco.upv.es>
*
* 	This file is part of MANET_Gateway.

    MANET_Gateway is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MANET_Gateway is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MANET_Gateway; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
* 
*
*/

#include "gatewayclient.h"

static bdaddr_t src_addr = *BDADDR_ANY;
static uint16_t role = BNEP_SVC_PANU;  /*local role*/
static uint16_t service = BNEP_SVC_NAP; /* Remote service */
static char netdev[16] = "bnep%d";



int RSA_security(int sk){
    
    RSA *key;
    FILE *pub_keyfile, *file = NULL;
    unsigned char *tmp = NULL;
     unsigned char array[256] = {0};
    datasend data;
    int bytes_read,numbytes,f;
    char buf[2048] = { 0 };
    
    memset(&data, 0, sizeof(datasend));
    if((file=fopen(KEY_FILE,"r"))==NULL){
	printf("no hay archivo de configuración\n");
	return 1;
    }
    else fscanf(file," %s\n",data.message);
    
   bytes_read = read(sk, buf, sizeof(buf));
   if( bytes_read > 0 ) {
       f=open(PUB_KEY,O_CREAT|O_TRUNC|O_WRONLY,0666);
       if(write(f,buf,bytes_read)<0){
	   printf("Error writing pub.key\n");
	   return 1;
       }
       close(f);
   }
   else{
       perror("I have not read anything from socket\n");
       return 1;
   }
      
   //read public key in file PUB_KEY
   if((pub_keyfile=fopen(PUB_KEY,"r")) == NULL){
       fprintf(stderr, "fopen()\n");
       exit(EXIT_FAILURE);
   }
   
   key = PEM_read_RSAPublicKey(pub_keyfile, NULL, NULL, NULL);
   fclose(pub_keyfile);  
   tmp = (unsigned char *)&data;
   RSA_public_encrypt(245, tmp, array, key, RSA_PKCS1_PADDING);
   RSA_free(key);

   //send encrypted key
   if ((numbytes = send(sk, array, 256, 0)) == -1) {
       perror("sendto");
       return 1;
   }

   return 0;
}



void cmd_name( int hdev, char *dev_name){
    
    int s = hci_open_dev(hdev);
    char name[248];
    
    if (s < 0) {
	printf("Can't open device hci%d. \n", hdev);
	exit(1);
    }
   
    if (hci_read_local_name(s, sizeof(name), name, 1000) < 0) {
	printf("Can't read local name on hci%d. \n", hdev);
	exit(1);
    }
    strcpy(dev_name, (char *)name);
}


void cmd_class( int hdev,  char *dev_class){
    
    static char *major_devices[] = { "Miscellaneous",
				     "Computer",
				     "Phone",
				     "LAN Access",
				     "Audio/Video",
				     "Peripheral",
				     "Imaging",
				     "Uncategorized" };
    int s = hci_open_dev(hdev);
    
    if (s < 0) {
	printf("Can't open device hci%d. \n", hdev);
	exit(1);
    }
    uint8_t cls[3];
    if (0 > hci_read_class_of_dev(s, cls, 1000)) {
	printf("Can't read class of device on hci%d. \n", hdev);
	exit(1);
    }
    if ((cls[1] & 0x1f) > sizeof(major_devices))
	printf("Invalid Device Class!\n");
    else{
	strcpy(dev_class, (char *)major_devices[cls[1] & 0x1f]);
    }
    
}



int localdescr(local_descr_t *servd,ifaces_t *iface){
    
    int ctl;
    int hdev;
    static struct hci_dev_info di;
    struct hci_dev_req dr;
    
    /* Open HCI socket  */
    if ((ctl = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI)) < 0) {
	perror("Can't open HCI socket.\n");
	return -1;
    }
    if((hdev = hci_get_route(NULL))<0){
	perror("Can't get route to device\n");
	close(ctl);
	return -1;
    }
    
    ioctl(ctl, HCIGETDEVINFO,(void *)&di);
    ba2str(&di.bdaddr,(char *) &servd->addr); 
    cmd_class(hdev, (char *) &servd->type);

    if((hdev = hci_get_route(NULL))<0){
	perror("Can't get route to device\n");
	close(ctl);
	return -1;
    }
   cmd_name(hdev, (char *)&servd->name);  
   iface->bluetooth_id=hdev;
   iface->blue_sock=ctl;
    
    //disable_scan
    dr.dev_id  = hdev;
    dr.dev_opt = SCAN_DISABLED;
    
    if( ioctl(ctl, HCISETSCAN, (unsigned long)&dr) < 0 ) {
	printf("Can't set scan mode on hci%d.\n", hdev);
	return -1;
    }
   
     //initiate bnep module
    if(bnep_init()<0)
        return -1;

    return 0; 
    
}
    

void enable_scan(int ctl, int hdev){
    
    struct hci_dev_req dr;
    
    dr.dev_id  = hdev;
    dr.dev_opt = SCAN_DISABLED;
    dr.dev_opt = SCAN_PAGE | SCAN_INQUIRY;
    
    if( ioctl(ctl, HCISETSCAN, (unsigned long)&dr) < 0 ) {
	printf("Can't set scan mode on hci%d.\n", hdev);
	exit(1);
    }
    
}


int manet_search(List *list, int len){
       
	inquiry_info *ii = NULL;
	uint32_t svc_uuid_int[] = { 0x0, 0x0, 0x0, 0x1BFF };
	uuid_t svc_uuid;
	int max_rsp, num_rsp;
	sdp_list_t *response_list = NULL, *search_list;
	sdp_session_t *session = 0;
	int dev_id, sock, flags;
	int i;
	char addr[19] = { 0 };
	char name_shown[248] = { 0 };
	int last_id,err;
	List l = NULL;
	Nodetype *nod;
	 
    
	dev_id = hci_get_route(NULL);
	sock = hci_open_dev( dev_id );
	if (dev_id < 0 || sock < 0) {
		perror("opening socket");
        	exit(1);
	}
	max_rsp = 255;
	flags = IREQ_CACHE_FLUSH;
	ii = (inquiry_info*)malloc(max_rsp * sizeof(inquiry_info));
	num_rsp = hci_inquiry(dev_id, len, max_rsp, NULL, &ii, flags);
	
	if(num_rsp < 0) 
		perror("hci_inquiry");

	// specify the UUID of the application we're searching for
	sdp_uuid128_create( &svc_uuid, &svc_uuid_int );
	search_list = sdp_list_append( NULL, &svc_uuid );
	i=0;
	for (; i < num_rsp; i++) {
		ba2str(&(ii+i)->bdaddr, addr);
	 	// connect to the SDP server running on the remote machine
		session = sdp_connect( BDADDR_ANY, &(ii+i)->bdaddr, SDP_RETRY_IF_BUSY );
		if(session){
			// get a list of service records that have UUID 0x1BFF
		               err=sdp_service_search_req(session, search_list,SDP_ATTR_REQ_RANGE,&response_list);
		               sdp_close(session);
			if(!err && sdp_list_len(response_list)){
				memset(name_shown, 0, sizeof(name_shown));
				if (hci_read_remote_name(sock, &(ii+i)->bdaddr, sizeof(name_shown),name_shown, 0) < 0)
					strcpy(name_shown,addr);
				char* serv_name= malloc(30);
				char* serv_addr = malloc(30);
				strcpy (serv_addr, addr);
				strcpy (serv_name, name_shown);
	    
				last_id=auto_insert(&l, serv_name, serv_addr);
				nod=ListNode(l, last_id);
				
			}
		}
	}
	*list=l;
	free(ii);
	close(sock);
	return 0;
}


int do_connect(char *baddr,conf_params *params){
    
    struct sockaddr_l2 l2a;
    struct l2cap_options l2o;
    int sk;
    socklen_t olen;
    char buf[MAXDATASIZE];
    char buf2[4];
    int numbytes;
    char *ip_address, *ip_gate,*ip_mask;
 
    printf("Connecting to %s\n", baddr);

    // allocate a socket
    sk = socket(AF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_L2CAP);

    if (sk < 0) {
	syslog(LOG_ERR, "Cannot create L2CAP socket. %s(%d)",strerror(errno), errno);
	return -1;
    }

    /* Setup L2CAP options according to BNEP spec */
    olen = sizeof(l2o);
    getsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &l2o, &olen);
    l2o.imtu = l2o.omtu = BNEP_MTU;
    setsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &l2o, sizeof(l2o));
    l2a.l2_family = AF_BLUETOOTH;
    
    /* Set local address */
    l2a.l2_psm =0;
    l2a.l2_bdaddr = src_addr;
    if (bind(sk, (struct sockaddr *) &l2a, sizeof(l2a))){
	printf("Bind failed\n");
	return -2;
    }
    l2a.l2_psm = htobs(BNEP_PSM);
    str2ba( baddr, &l2a.l2_bdaddr );
	
    // connect to server
    if (connect(sk, (struct sockaddr *) &l2a, sizeof(l2a))!=-1){
	numbytes = read(sk, buf2, sizeof(buf2));
	if( numbytes > 0 ) {
	    if(strncmp(buf2,"SECY",4)==0){ //with security
		if(RSA_security(sk)==1){
		    close(sk);	
		    return -5;
		}
		numbytes = read(sk, buf2, sizeof(buf2));
		if( numbytes > 0 ) {
		    if(strncmp(buf2,"AUTN",4)==0){
			close(sk);
			return -6;
		    }
		}
		else{
		    close(sk);
		    return -3;		
		}
	    }

	}
	else{
	    close(sk);
	    return -3;
	}
	
	if((numbytes=recv(sk,buf,MAXDATASIZE-1,0))==-1){
	    close(sk);
	    return -3;			
	}
	buf[numbytes]='\0';
	char s[4]=" \n\t";
	char *ptr;
	ptr=strtok(buf,s); 
	ip_gate=ptr;
	params->ip_gate.sin_addr.s_addr=inet_addr(ip_gate);
	ptr=strtok(NULL,s);
	ip_address=ptr;
	params->ip_address.sin_addr.s_addr=inet_addr(ip_address);
	ptr=strtok(NULL,s);
	ip_mask=ptr;
	params->ip_mask.sin_addr.s_addr=inet_addr(ip_mask);
			
	if(bnep_create_connection(sk, role, service, netdev)) {	
	    return -4;
	}
    }
    else {
	return -3;
    }
	
    // close connection
    close(sk);
    return 0;
}



