/*
*  C Implementation: Gateway_Client
*
* Copyright (C) -2007  MªCruz Olivas Delgado <maoldel@teleco.upv.es>
*
* 	This file is part of MANET_Gateway.

    MANET_Gateway is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MANET_Gateway is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MANET_Gateway; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
* 
*
*/

using namespace std;
#include <string>
#include "mainwindow.h"



MainWindow::MainWindow(QWidget *parent, const char *name)
    : QMainWindow(parent, name)
{
    setCaption("Gateway Client - 1.0");
    setIcon(QPixmap::fromMimeSource("icons/grc.jpg"));
    
    grclogoLabel = new QLabel(this);
    bluetoothlogoLabel = new QLabel(this);
    grclogoLabel->setPixmap(QPixmap::fromMimeSource("icons/grc.jpg"));
    bluetoothlogoLabel->setPixmap(QPixmap::fromMimeSource("icons/bluesin.jpeg"));
   
    createMenuBar();
    
    localdeviceButtonGroup = new QButtonGroup( 2, QGroupBox::Horizontal, tr("Local Device"),this);

    (void)new QLabel(tr( "Address"), localdeviceButtonGroup );
    addressvalue = new QLabel( "", localdeviceButtonGroup );
    addressvalue->setFrameStyle( QFrame::Panel | QFrame::Sunken );

    (void)new QLabel(tr("Name"), localdeviceButtonGroup );
    namevalue = new QLabel( "", localdeviceButtonGroup );
    namevalue->setFrameStyle( QFrame::Panel | QFrame::Sunken );

   (void)new QLabel(tr("Type"), localdeviceButtonGroup );
    typevalue = new QLabel( "", localdeviceButtonGroup );
    typevalue->setFrameStyle( QFrame::Panel | QFrame::Sunken );


    serversLabel = new QLabel(tr("Servers"), this);
    tasksLabel = new QLabel(tr("Tasks"), this);
    scantimeLabel = new QLabel(tr("Scan time"),this);

    scantimeSpinBox = new MySpinBox(128,2560,128,this,0);
    scantimeSpinBox->setValue(512);

    serversText = new QListBox(this);
   
    serversText->setMinimumHeight(140);
    serversText->setMinimumWidth(160);
    tasksText = new QListBox(this);
    tasksText->setMinimumHeight(140);
    tasksText->setMinimumWidth(180);

    searchButton = new QPushButton(tr("Search"),this);
    connectButton = new QPushButton(tr("Connect"),this);
    disconnectButton = new QPushButton(tr("Disconnect"),this);    
    exitButton = new QPushButton(tr("EXIT"),this);
    keyButton= new QPushButton(this);
    keyButton->setPixmap(QPixmap::fromMimeSource("icons/llave.jpeg"));
    
    disconnectButton->setEnabled(false);
    exitButton->setMinimumHeight(50);
    exitButton->setMinimumWidth(300);
    keyButton->setMaximumHeight(40);
    keyButton->setMaximumWidth(80);
    searchButton->setMinimumHeight(50);
    connectButton->setMinimumHeight(35);
    disconnectButton->setMinimumHeight(35);

    topLayoutmedium = new QVBoxLayout;
    topLayoutmedium->addStretch(0);
    topLayoutmedium->addWidget(scantimeLabel);
    topLayoutmedium->addWidget(scantimeSpinBox);
    topLayoutmedium->addStretch(0);
  
    topLayout = new QHBoxLayout;
    topLayout->addLayout(topLayoutmedium);
    topLayout->addWidget(searchButton);
    topLayout->addStretch(0);
    topLayout->addWidget(localdeviceButtonGroup);
    topLayout->addStretch(0);
    topLayout->addWidget(bluetoothlogoLabel);
      
    mediumLayoutleft = new QVBoxLayout;
    mediumLayoutleft->addWidget(serversLabel);
    mediumLayoutleft->addWidget(serversText);
    
    mediumLayoutmedium = new QVBoxLayout;
    mediumLayoutmedium->addWidget(connectButton);
    mediumLayoutmedium->addWidget(disconnectButton);

    mediumLayoutright = new QVBoxLayout;
    mediumLayoutright->addWidget(tasksLabel);
    mediumLayoutright->addWidget(tasksText);
    
    mediumLayout = new QHBoxLayout;
    mediumLayout->addLayout(mediumLayoutleft);
    mediumLayout->addLayout(mediumLayoutmedium);
    mediumLayout->addLayout(mediumLayoutright);
    
    bottomLayoutkey= new QHBoxLayout;
    bottomLayoutkey->addStretch(0);
    bottomLayoutkey->addWidget(keyButton);
    
    bottomLayoutright=new QVBoxLayout;
    bottomLayoutright->addLayout(bottomLayoutkey);
    bottomLayoutright->addWidget(exitButton);
    
    bottomLayout = new QHBoxLayout;
    bottomLayout->addWidget(grclogoLabel);
    bottomLayout->addLayout(bottomLayoutright);
    bottomLayout->addStretch(0);

    mainLayout = new QVBoxLayout(this);    
    mainLayout->setMargin(10);
    mainLayout->setSpacing(10);
    mainLayout->addSpacing(20);
    mainLayout->addLayout(topLayout);
    mainLayout->addLayout(mediumLayout);
    mainLayout->addLayout(bottomLayout);
    mainLayout->addWidget(statusBar());
    statusBar();
   
    statusBar()->message(tr("Loading..."),1000);  // Initial message
    
    statusBar()->message(tr("Done."), 3000); // Final message for 2 seconds
       
    QObject::connect(exitButton,SIGNAL(clicked()),this,SLOT(close()));
    QObject::connect(searchButton,SIGNAL(clicked()),this,SLOT(do_inquiry()));
    QObject::connect(connectButton,SIGNAL(clicked()),this,SLOT(gate_connect()));
    QObject::connect(disconnectButton,SIGNAL(clicked()),this,SLOT(gate_disconnect()));
    QObject::connect(keyButton,SIGNAL(clicked()),this,SLOT(enter_key()));

    if(init()<0){
         begin=0;
         MainWindow::close();
    }
    else begin=1;
        
}

void MainWindow::createMenuBar(){
    
    //Actins
    aboutAct = new QAction(tr("&About"), 0, this);
    aboutAct->setStatusTip(tr("Show the application's About box"));
    connect(aboutAct, SIGNAL(activated()), this, SLOT(about()));

    aboutQtAct = new QAction(tr("About &Qt"), 0, this);
    aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
    connect(aboutQtAct, SIGNAL(activated()), qApp, SLOT(aboutQt()));
    
    //add actions to helpMenu
    helpMenu = new QPopupMenu(this);
    aboutAct->addTo(helpMenu);
    aboutQtAct->addTo(helpMenu);
    
    //add helpMenu to menuBar
    menuBar = new QMenuBar(this);
    menuBar->setSeparator( QMenuBar::Never);
    menuBar->insertSeparator(0);                      		 	  	   
    menuBar->insertItem(tr("&Help"),helpMenu);
}

void MainWindow::about(){
    QMessageBox::about(this, tr("About Spreadsheet"),
            tr("<h2>Gateway Client  1.0</h2>"
               "<p>End of Degree Project"
               "<p>Mª Cruz Olivas Delgado "));
}

int MainWindow::init(){ 
 
    if(descr_local_device()<0){
	QMessageBox::critical( this, "Gateway Client - 1.0", "Bluetooth device not Ready ", QMessageBox::Ok, 0);
	return -1;
    }
        
    return 0;
}

void MainWindow::closeEvent( QCloseEvent* ce ){

    if(!begin){
	exit(1);
    }
    switch( QMessageBox::information( this,"Gateway Client - 1.0 ","Are you sure you want to exit?", "Yes","Cancel", 0, 1 ) ){
	  case 0:
	      disconThread.stop();
	      enable_scan(iface.blue_sock,iface.bluetooth_id);
	      ce->accept();
	      break;
	  case 1:
	  default: // just for sanity
	      ce->ignore();
	      break;
	  }
}


int MainWindow::descr_local_device(){
    local_descr_t clientdes;
   
    if(localdescr(&clientdes,&iface)==-1)
	return -1;
    addressvalue->setText(QString::QString (clientdes.addr));
    namevalue->setText(QString::QString (clientdes.name));
    typevalue->setText(QString::QString (clientdes.type));
   
    return 0;
}

void MainWindow::do_inquiry(){
    Nodetype *node;
    int scan_time;
    
    serversText->clear();
    tasksText->clear();
    
    scan_time=int((scantimeSpinBox->value())/128);
    statusBar()->message(tr("Inquiry ..."),25000);
    manet_search(&list, scan_time);


    statusBar()->message(tr("Inquiry Finished"),2000);
    
    node=list;
    if(EmptyList(list)){
	serversText->insertItem(tr("No Server found"));
    }
    else {
	while(node) {
	    serversText->insertItem(node->name);
	    node = node->next;
	}
	serversText->setSelectionMode(QListBox::Single);
    }

}

void MainWindow::gate_connect(){
    
   Nodetype*nod=NULL;
    int chose,er;
    int sys;
    unsigned int i=0;
    char cad[255];
    char *ip_address,*ip_gate,*ip_mask;
    bool selec=FALSE;
    
    if((chose=serversText->currentItem()<0) || (serversText->currentText()=="No Server found")){
	perror("No server Selected\n");
	return;
    }
    else{
	for (i;i<serversText->count();i++){
	   
	    if(serversText->isSelected(i)){
		nod = ListNode(list,i+1);
		serversText->setSelectionMode(QListBox::NoSelection);
		selec=TRUE;
		tasksText->clear();
	    }
                }	
	
	if(selec){
	     tasksText->clear();
	     tasksText->insertItem(tr("Connect to server..."));

	    er=do_connect(nod->string_addr,&params);
		
	    if(er==-1||er==-2||er==-3||er==-4||er==-5||er==-6){
	                tasksText->insertItem(QPixmap::fromMimeSource("alt.gif"), tr("Bnep0 not created!"));
		if(er==-1) tasksText->insertItem(QPixmap::fromMimeSource("icons/alt.gif"),tr("Cannot create L2CAP socket"));
		if(er==-2) tasksText->insertItem(QPixmap::fromMimeSource("icons/alt.gif"),tr("Bind failed"));
		if(er==-3) tasksText->insertItem(QPixmap::fromMimeSource("icons/alt.gif"),tr("Connect not accepted"));
		if(er==-4) tasksText->insertItem(QPixmap::fromMimeSource("icons/alt.gif"),tr("Cannot create bnep connection"));
		if(er==-5) tasksText->insertItem(QPixmap::fromMimeSource("icons/alt.gif"),tr("Server with security"));
		if(er==-6) tasksText->insertItem(QPixmap::fromMimeSource("icons/alt.gif"),tr("Authentication failed"));
		tasksText->insertItem(tr("Try to reconnect"));
		serversText->setSelectionMode(QListBox::Single);
		serversText->clearSelection();
	    }

	    else{
	                er=0;
		tasksText->insertItem(QPixmap::fromMimeSource("icons/ok.gif"),tr("Bnep0 successfully created!"));
		ip_address=inet_ntoa(params.ip_address.sin_addr);
		sprintf(cad,"ifconfig bnep0 %s",ip_address);
		sys=system(cad);
		if((sys!=-1) & (sys!=127)){
		    sprintf(cad,"IP address %s",ip_address);
		    tasksText->insertItem(QPixmap::fromMimeSource("icons/ok.gif"),tr(cad));
		    connectButton->setEnabled(false);
		}
		else {
		    tasksText->insertItem(QPixmap::fromMimeSource("icons/alt.gif"),tr("Error in the asigment of IP address"));
		    er=1;
		}
		ip_mask=inet_ntoa(params.ip_mask.sin_addr);
		sprintf(cad,"ifconfig bnep0 netmask %s ",ip_mask);
		sys=system(cad);
		if((sys!=-1) & (sys!=127)){
		    sprintf(cad,"Netmask %s configured",ip_mask);
		    tasksText->insertItem(QPixmap::fromMimeSource("icons/ok.gif"),tr(cad));
		 }
		else {
		    tasksText->insertItem(QPixmap::fromMimeSource("icons/alt.gif"),tr("Error in the asigment of Gateway address"));
		    er=1;
		}
		ip_gate=inet_ntoa(params.ip_gate.sin_addr);
		sprintf(cad,"route add default gw %s dev bnep0",ip_gate);
		sys=system(cad);
		if((sys!=-1) & (sys!=127)){
		    sprintf(cad,"Gateway address %s",ip_gate);
		    tasksText->insertItem(QPixmap::fromMimeSource("icons/ok.gif"),tr(cad));
		    tasksText->insertItem(QPixmap::fromMimeSource("icons/ok.gif"),tr("Routing table actualized"));
		}
		else {
		    tasksText->insertItem(QPixmap::fromMimeSource("icons/alt.gif"),tr("Error in the asigment of Gateway address"));
		    er=1;
		}
		
		if(er==0) {
		    bluetoothlogoLabel->setPixmap(QPixmap::fromMimeSource("icons/bluecon.jpeg"));
		    tasksText->insertItem(tr("Connection succesful!"));
		    disconnectButton->setEnabled(true);
		    searchButton->setEnabled(false);
		    disconThread.setTargetWidget(this);
		    disconThread.start();    
		}
		else if (er==1) {
		    tasksText->insertItem(QPixmap::fromMimeSource("icons/alt.gif"),tr("Error stablishing configuration parameters"));
		    tasksText->insertItem(tr("Try to reconnect"));
		    serversText->setSelectionMode(QListBox::Single);
		    serversText->clearSelection();
		    bnep_kill_all_connections();
		 }
	    }
	}
    }


}

void MainWindow::gate_disconnect(){
    
       if(bnep_kill_all_connections()==-1) statusBar()->message(tr("Failed to remove connection"),5000);
       else{
	   disconThread.stop();
	   tasksText->clear();
	   tasksText->insertItem(QPixmap::fromMimeSource("icons/ok.gif"),tr("The disconnection "));
	   tasksText->insertItem(tr("      was succesfull"));
	   searchButton->setEnabled(true);
	   connectButton->setEnabled(true);
	   disconnectButton->setEnabled(false);
	   bluetoothlogoLabel->setPixmap(QPixmap::fromMimeSource("icons/bluesin.jpeg"));
	   serversText->setSelectionMode(QListBox::Single);
	   serversText->clearSelection();
       
       }
     
}


void MainWindow::enter_key(){
    
    // put key in KEY_FILE
    FILE *file;
    bool ok = FALSE;
    
    QString key = QInputDialog::getText(  tr("KEY"),tr( "Enter server's key:" ), QLineEdit::Password, QString::null, &ok, this );

    if ( !key.isEmpty() && ok ){
	file=fopen(KEY_FILE,"w");
	fprintf(file,key.ascii());
	fclose(file);
    }
}


void MainWindow::customEvent(QCustomEvent *event){    
    
    if ((int)event->type() == TransactionStart) {
	bluetoothlogoLabel->setPixmap(QPixmap::fromMimeSource("icons/bluesin.jpeg"));
	tasksText->clear();
	tasksText->insertItem(QPixmap::fromMimeSource("icons/alt.gif"),tr("The connection has been")); 
	tasksText->insertItem(tr("    interrupted externally"));
	tasksText->insertItem(tr("Try to reconnect"));
	disconnectButton->setEnabled(false);
	searchButton->setEnabled(true);
	connectButton->setEnabled(true);
	
    } else {
    QMainWindow::customEvent(event);
    }

}
