#ifndef GATEWAYCLIENT_H
#define GATEWAYCLIENT_H

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/socket.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <syslog.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>

//Bluetooth Part Headers
#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <bluetooth/rfcomm.h>
#include <bluetooth/sdp.h>
#include <bluetooth/sdp_lib.h>

//RSA security
#include <openssl/rsa.h>
#include <openssl/pem.h>

//List Treatment
#include "listsc.h"
#include "bnepc.h"


#define VERSION "1.0"
#define CONF_FILE "files/conf.txt"
#define PRIV_KEY "files/priv.key"
#define PUB_KEY "files/pub.key"
#define KEY_FILE "files/pin.key"
#define MAC_FILE "files/authorized_macs.txt"
#define IP_LAST "/tmp/last_ip.txt"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif
    
    //int terminate;
    
    typedef struct{
	char interface[20];
	int bluetooth_id;
	int blue_sock;
    } ifaces_t;
    
    typedef struct{
	char addr[20];
	char name[20];
	char type[20];
    } local_descr_t;
    
  typedef struct {
	char interface[10];
	int max_hosts;
	char ip_gate[15];
	char ip_mask[15];
	int seculevel;	
  }conf_params;
    
   typedef struct {
        char message[256];
   }datarec;
   
   static struct sockaddr_in dest;
   
   //security routines
    int RSA_write_files();
    int RSA_security(int nsk);
    int check_MAC(char *str);
       
  //bluetooth routines
    int get_max_clients();
    void cmd_name(int hdev, char *dev_name);
    void cmd_class(int hdev,  char *dev_class);
    int localdescr(local_descr_t *servd,ifaces_t *iface);
    int add_service();
    int del_manet_service();
    int do_listen();
       
    //other routines
    void configure_params();
    char * suma_ip (int );
    int get_last_ip();
    void set_last_ip (char* ip);
    int any_client_served();
  

#ifdef __cplusplus
}
#endif


#endif
