/*
  *  C Implementation: Gateway_Server
*
* Copyright (C) -2007  MªCruz Olivas Delgado <maoldel@teleco.upv.es>
*
  *
  *	This file is part of MANET_Gateway.

    MANET_Gateway is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MANET_Gateway is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MANET_Gateway; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
    
  */


using namespace std;
#include <string>
#include "mainwindow.h"

  
MainWindow::MainWindow(QWidget *parent, const char *name)
    : QMainWindow(parent, name)
{
    clients=0;
    setCaption("Gateway Server - 1.0");
    setIcon(QPixmap::fromMimeSource("icons/grc.jpg"));

    bluetoothlogoLabel = new QLabel(this);
    bluetoothlogoLabel->setPixmap(QPixmap::fromMimeSource("icons/bluesin.jpeg"));
    grclogoLabel = new QLabel(this);
    grclogoLabel->setPixmap(QPixmap::fromMimeSource("icons/grc.jpg"));
    clientsnumberLabel = new QLabel(tr("Number of clients:"),this);
    clientsvalueLabel =    new QLabel("0",this);
    barLabel =    new QLabel("/",this);
    maxclientsLabel =    new QLabel("0",this);
    
     createMenuBar();
    
    localdeviceButtonGroup = new QButtonGroup( 2, QGroupBox::Horizontal, tr("Local Device"),this);

    (void)new QLabel(tr( "Address"), localdeviceButtonGroup );
    addressvalue = new QLabel( "", localdeviceButtonGroup );
    addressvalue->setFrameStyle( QFrame::Panel | QFrame::Sunken );

    (void)new QLabel(tr("Name"), localdeviceButtonGroup );
    namevalue = new QLabel( "", localdeviceButtonGroup );
    namevalue->setFrameStyle( QFrame::Panel | QFrame::Sunken );

    (void)new QLabel(tr("Type"), localdeviceButtonGroup );
    typevalue = new QLabel( "", localdeviceButtonGroup );
    typevalue->setFrameStyle( QFrame::Panel | QFrame::Sunken );
   
    editorText = new QListBox(this);
   
    editorText->setMinimumHeight(180);
    editorText->setMinimumWidth(190);
    editorText->setFocusPolicy( QWidget::StrongFocus );

    startorstopButton = new QPushButton(tr("START SERVER"),this);    
    parametersButton = new QPushButton(tr("Parameters"),this);
    exitButton = new QPushButton(tr("EXIT"),this);
    delallButton= new QPushButton(tr("Delete All Connections"),this);
    delconButton= new QPushButton(tr("Delete Connection"),this);    
    exitButton->setMinimumHeight(30);
    delallButton->setEnabled(false);
    delconButton->setEnabled(false);
    
    topLayoutleft = new QVBoxLayout;
    topLayoutleft->addWidget(startorstopButton);
    topLayoutleft->addWidget(parametersButton);

    topLayout = new QHBoxLayout;
    topLayout->addLayout(topLayoutleft);
    topLayout->addStretch(1);
    topLayout->addWidget(localdeviceButtonGroup);
    topLayout->addStretch(1);
    topLayout->addWidget(bluetoothlogoLabel);

    mediumLayoutleft= new QVBoxLayout;
   
    mediumLayoutleft->addWidget(delallButton);
    mediumLayoutleft->addWidget(delconButton);

    mediumLayout = new QHBoxLayout;
    mediumLayout->addLayout(mediumLayoutleft);
    mediumLayout->addWidget(editorText);
    
    bottomLayoutnum = new QHBoxLayout;
    bottomLayoutnum->addStretch(1);
    bottomLayoutnum->addWidget(clientsnumberLabel);
    bottomLayoutnum->addWidget(clientsvalueLabel);
    bottomLayoutnum->addWidget(barLabel);
    bottomLayoutnum->addWidget(maxclientsLabel);
    
    bottomLayoutrigth = new QVBoxLayout;
    bottomLayoutrigth->addSpacing(20);
    bottomLayoutrigth->addWidget(exitButton);
    bottomLayoutrigth->addStretch(1);
    bottomLayoutrigth->addLayout(bottomLayoutnum);

    bottomLayout = new QHBoxLayout;
    bottomLayout->addWidget(grclogoLabel);
    bottomLayout->addSpacing(20);
    bottomLayout->addLayout(bottomLayoutrigth);
    
    mainLayout = new QVBoxLayout(this);
    mainLayout->setMargin(10);
    mainLayout->setSpacing(10);
    mainLayout->addSpacing(20);
    mainLayout->addLayout(topLayout);
    mainLayout->addSpacing(20);
    mainLayout->addLayout(mediumLayout);
    mainLayout->addSpacing(20);
    mainLayout->addLayout(bottomLayout);
    
    parameterDialog = 0;
    
    QObject::connect(exitButton,SIGNAL(clicked()),this,SLOT(close()));
    QObject::connect(parametersButton,SIGNAL(clicked()),this,SLOT(parameters()));  
    QObject::connect(startorstopButton,SIGNAL(clicked()),this,SLOT(startorstopServer()));  
    QObject::connect(delallButton,SIGNAL(clicked()),this,SLOT(delallconnections()));  
    QObject::connect(delconButton,SIGNAL(clicked()),this,SLOT(delconnection()));  
    
    
    if(descr_local_device()<0){
         begin=FALSE;
         MainWindow::close();
    }
    else{
	 begin=TRUE; 
	 configure_params();    
	 add_service();
    }
}


void MainWindow::createMenuBar()
{
    //Actins
    aboutAct = new QAction(tr("&About"), 0, this);
    aboutAct->setStatusTip(tr("Show the application's About box"));
    connect(aboutAct, SIGNAL(activated()), this, SLOT(about()));

    aboutQtAct = new QAction(tr("About &Qt"), 0, this);
    aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
    connect(aboutQtAct, SIGNAL(activated()), qApp, SLOT(aboutQt()));
    
    //add actions to helpMenu
    helpMenu = new QPopupMenu(this);
    aboutAct->addTo(helpMenu);
    aboutQtAct->addTo(helpMenu);
    
    //add helpMenu to menuBar
    
    menuBar = new QMenuBar(this);
    menuBar->setSeparator( QMenuBar::Never);
    menuBar->insertSeparator(0);                      		 	  	   
    menuBar->insertItem(tr("&Help"),helpMenu);

}

void MainWindow::about(){
    QMessageBox::about(this, tr("About Gateway Server"),
            tr("<center><h3><font color=blue>Gateway Server 1.0</font></h3>"
             	            "<p><h4><center>Author: Maricruz Olivas Delgado </h4></p>"
               "<p> <center>Direct by Miguel Mateo and Carlos Calafate</p>"
	         "<p><i><center>Polytechnic University of Valencia</center></i></p>"
	  "<p><i>Grupo de Redes de Computadores: </i> <A HREF=http://www.grc.upv.es> www.grc.upv.es</A></p> "
	   "<p>  <p><right> <h5>Distributed under GPL License</right></h5></p>" ));
 }



int MainWindow::init(){ 
  
    max_clients=get_max_clients();
    maxclientsLabel->setNum(max_clients);
    listenThread.setTargetWidget(this);
    listenThread.start();    
    
    return 0;
}

void MainWindow::closeEvent( QCloseEvent* ce ){

    if(!begin){
	exit(1);
    }
    switch( QMessageBox::information( this,"Gateway Client - 1.0 ","Are you sure you want to exit?", "Yes","Cancel", 0, 1 ) ){
	  case 0:
	      listenThread.stop();
	      countThread.stop();
	      if(bnep_count_connections()>0){
		  bnep_kill_all_connections();
		  system("ifconfig pan0 down");
		  system("brctl delbr pan0");
	      }
	      del_manet_service();
	      ce->accept();
	      break;
	  case 1:
	  default: // just for sanity
	      ce->ignore();
	      break;
	  }
}

int MainWindow::descr_local_device(){
    
    local_descr_t clientdes;
   
    if(localdescr(&clientdes,&iface)==-1)
	return -1;

    addressvalue->setText(QString::QString (clientdes.addr));
    namevalue->setText(QString::QString (clientdes.name));
    typevalue->setText(QString::QString (clientdes.type));
   
    return 0;
}

void MainWindow::startorstopServer(){
    if(startorstopButton->text()=="START SERVER")
    {
	countThread.stop();	 	
	startorstopButton->setText("STOP SERVER");
	parametersButton->setEnabled(false);
	init();
    }
    else{
	listenThread.stop();
	startorstopButton->setText("START SERVER");
	countThread.setTargetWidget(this);
	countThread.start();
    }
}



void MainWindow::customEvent(QCustomEvent *event){    
    
    if ((int)event->type() == TransactionStart) {
	clients=bnep_count_connections();
	if(clients==0) {
	    delconButton->setEnabled(false);
	    delallButton->setEnabled(false);
	}
	if(clients==1) {
	    bluetoothlogoLabel->setPixmap(QPixmap::fromMimeSource("icons/blue.jpeg"));
	}
	clientsvalueLabel->setNum(clients);
	showconnections();
	
    } else {
	QMainWindow::customEvent(event);
    }
}

void MainWindow::showconnections(){
    int i;
    char cad[255];
    Nodetype *node;
    editorText->clear();
    i=bnep_show_connections(&list);
    if (i==-1) editorText->insertItem("failed to get connection list");
    else if (i) {
	delconButton->setEnabled(true);
	delallButton->setEnabled(true);
	node=list;
	ShowList(list);
	while(node) {
	    int i=1;
	    sprintf(cad,node->device);
	    strcat(cad," :    ");
	    strcat(cad,node->string_addr);
	    editorText->insertItem(cad,i);
	    node = node->next;
	    i++;
	}	
   }
    
    else{
	editorText->insertItem("There isn't any device conected");
	if(startorstopButton->text()=="START SERVER")  parametersButton->setEnabled(true);
	bluetoothlogoLabel->setPixmap(QPixmap::fromMimeSource("icons/bluesin.jpeg"));
    }
}

void MainWindow::delconnection(){
     Nodetype *nod;
    unsigned int i=0;
          
    for (i;i<editorText->count();i++){
	if(editorText->isSelected(i)){
		nod =ListNode(list,i+1);
		bnep_kill_connection((uint8_t *) strtoba(nod->string_addr));
		Erase(&list, nod->id);
	}
    }
	
    clients=bnep_count_connections();
    clientsvalueLabel->setNum(clients);
    if(clients==0)  {
	delconButton->setEnabled(false);
	delallButton->setEnabled(false);
	bluetoothlogoLabel->setPixmap(QPixmap::fromMimeSource("icons/bluesin.jpeg"));
    }
    showconnections();
}

void MainWindow::delallconnections(){
     int i=bnep_kill_all_connections();
     if(!i){
	 delconButton->setEnabled(false);
	 delallButton->setEnabled(false);
	 bluetoothlogoLabel->setPixmap(QPixmap::fromMimeSource("icons/bluesin.jpeg"));
  
	 showconnections();
     }
     else QMessageBox::information( this, "Gateway Client - 1.0 ", "Sorri, can't delete all connections");
   
}

void MainWindow::parameters(){
    
    if (!parameterDialog) {
        parameterDialog = new ParameterDialog(this);
    }

    
    parameterDialog->show();
    parameterDialog->raise();
    parameterDialog->setActiveWindow();

}

