/*
  *  C Implementation: Gateway_Server
*
* Copyright (C) -2007  MªCruz Olivas Delgado <maoldel@teleco.upv.es>
*
  *	This file is part of MANET_Gateway.

    MANET_Gateway is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    MANET_Gateway is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MANET_Gateway; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
    
  */

#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include "parameterdialog.h"

ParameterDialog::ParameterDialog(QWidget *parent, const char *name) 
    : QDialog(parent, name)
{
    setCaption("MANET Parameters");
    
    securityLabel = new QLabel("Security",this);
    pinLabel = new QLabel("KEY",this);
    ipgateLabel = new QLabel("IP Gate",this);
    ipmaskLabel = new QLabel("IP Mask",this);
    maxhostsLabel = new QLabel("Max Hosts",this);
      
    securityComboBox = new QComboBox(this);
    pinLineEdit = new QLineEdit(this);
    pinLineEdit->setReadOnly(true);
    ipgateLineEdit = new QLineEdit(this);
    ipmaskLineEdit = new QLineEdit(this);
    maxhostsComboBox = new QComboBox(this);
    okButton = new QPushButton("Ok",this);
    cancelButton = new QPushButton("Cancel",this);
    
    securityComboBox->insertItem("NONE");
    securityComboBox->insertItem("KEY");
    securityComboBox->insertItem("Allowed MACs");
    securityComboBox->insertItem("KEY+MAC");
    
    maxhostsComboBox->insertItem("1");
    maxhostsComboBox->insertItem("2");
    maxhostsComboBox->insertItem("3");
    maxhostsComboBox->insertItem("4");
    maxhostsComboBox->insertItem("5");
    maxhostsComboBox->insertItem("6");
    maxhostsComboBox->insertItem("7");
 
    securityLayout = new QHBoxLayout;
    securityLayout->addWidget(securityLabel);
    securityLayout->addWidget(securityComboBox);
    
    pinLayout = new QHBoxLayout;
    pinLayout->addWidget(pinLabel);
    pinLayout->addWidget(pinLineEdit);

    ipgateLayout = new QHBoxLayout;
    ipgateLayout->addWidget(ipgateLabel);
    ipgateLayout->addWidget(ipgateLineEdit);

    ipmaskLayout = new QHBoxLayout;
    ipmaskLayout->addWidget(ipmaskLabel);
    ipmaskLayout->addWidget(ipmaskLineEdit);
    
    maxhostsLayout = new QHBoxLayout;
    maxhostsLayout->addStretch(1);
    maxhostsLayout->addWidget(maxhostsLabel);
   
    maxhostsLayout->addWidget(maxhostsComboBox);

    buttonLayout = new QHBoxLayout;
    buttonLayout->addWidget(okButton);
    buttonLayout->addWidget(cancelButton);
    
    mainLayout = new QVBoxLayout(this);
    mainLayout->setMargin(10);
    mainLayout->setSpacing(5);
    mainLayout->addLayout(ipgateLayout);    
    mainLayout->addLayout(ipmaskLayout);
    mainLayout->addLayout(maxhostsLayout);
    mainLayout->addLayout(securityLayout);
    mainLayout->addLayout(pinLayout);
    mainLayout->addLayout(buttonLayout);    
    
    QObject::connect(cancelButton,SIGNAL(clicked()),this,SLOT(close()));
    QObject::connect(okButton,SIGNAL(clicked()),this,SLOT(submit()));
    QObject::connect(securityComboBox,SIGNAL(activated(int)),this,SLOT(activatepin(int)));
}

void ParameterDialog::submit(){
    QString *secu;
    QString *pin;
    QString *ipg=NULL;
    QString *ipm=NULL;
    QString *max_h;
 
    FILE *arch,*key;    
    int max_hosts;
    int level;
    char *aux;
    bool error=FALSE;
    
    secu=&securityComboBox->currentText();
    
    if(strncmp(secu->ascii(),"NONE",4)==0) level=0;
    else if(strncmp(secu->ascii(),"Allowed MACs",12)==0) level=1;
    else if(strncmp(secu->ascii(),"KEY",10)==0) level=2;
    else level=3;
        
    if((level==2)||(level==3)){
	if(pinLineEdit->edited()==TRUE){
	    pin=&pinLineEdit->text();
	    key=fopen(KEY_FILE,"w");
	    fprintf(key,pin->ascii());
	    fclose(key);
	}
	else{
	    QMessageBox::information( this, "Gateway Client - 1.0 ", "You have to put the key.\n");
	    error=TRUE;
	}
	
    }
    if((level==1)||(level==3)){
	//FILE *arch;
	QMessageBox::information( this, "Gateway Client - 1.0 ", "You have to put the authorized BD_ADDR's in a file called 'authorized_macs.txt', in 'files' directory\n");
	//arch=fopen(MAC_FILE,"a");
	//fclose(arch);
	    	
    }
    if(ipgateLineEdit->edited()==FALSE){
	if(ipmaskLineEdit->edited()==FALSE) {
	    QMessageBox::information( this, "Gateway Client - 1.0 ", "You have to put the ip mask and the ip address.\n");
	    error=TRUE;
	}
	else{
	    QMessageBox::information( this, "Gateway Client - 1.0 ", "You have to put the ip address.\n");
	    error=TRUE;
	}
    }
    else if(ipmaskLineEdit->edited()==FALSE){
	 QMessageBox::information( this, "Gateway Client - 1.0 ", "You have to put the ip mask.\n");
	 error=TRUE;
    }
    else {
	ipg=&ipgateLineEdit->text();
	if(inet_addr(ipg->ascii())==-1){
	    QMessageBox::information( this, "Gateway Client - 1.0 ", "The ip gate is wrong.\n");
	    error=TRUE;
	}
	ipm=&ipmaskLineEdit->text();
	if(inet_addr(ipm->ascii())==-1){
	    QMessageBox::information( this, "Gateway Client - 1.0 ", "The ip mask is wrong.\n");
	    error=TRUE;
	}
    }
    
    max_h=&maxhostsComboBox->currentText();
    
    if(error==FALSE) {
	arch = fopen(CONF_FILE,"w");       
	aux=(char*)max_h->ascii();
	max_hosts=atoi(aux);
	fprintf(arch,"maximum_hosts=%d\nip_gate=%s\nip_mask=%s\nsecurity_level=%d",max_hosts,ipg->ascii(),ipm->ascii(),level);      
  	fclose(arch);                         
	configure_params(); 
	close();
    }
    
}

void ParameterDialog::activatepin(int i){
    
    switch ( i ) {
    case 0:
        pinLineEdit->setReadOnly(true);
        break;
    case 1:
         pinLineEdit->setReadOnly(false);
         pinLineEdit->setEchoMode( QLineEdit::Password);
        break;
    case 2:
         pinLineEdit->setReadOnly(true);
        break;
    case 3:
         pinLineEdit->setReadOnly(false);
         pinLineEdit->setEchoMode( QLineEdit::Password);
        break;
    }
   pinLineEdit->setFocus();
    
}

